#include <stdint.h>
#include <stdbool.h>
#include <stdlib.h>
#include <stdio.h>

#include "gps.h"
#include "mstimer.h"

#define AVERAGE_SHIFT_DIVISOR 10
#define TIMEOUT_MS          3000
#define CONFIDENCE_OK         15

static int confidence = 0;
bool NmeaFixIsStable() { return confidence == CONFIDENCE_OK; }

int  NmeaLat; // One degree is represented as 10 million (  90 ==   900,000,000) so one metre is represented by 100
int  NmeaLng; // One degree is represented as 10 million ( 180 == 1,800,000,000) so one metre is represented by 100
int  NmeaHgt; // One metre  is represented as  1 million (2100 == 2,100,000,000)

uint32_t nmeaLatAvLow = 0x80000000; //Holds the parts below the limits of an int
uint32_t nmeaLngAvLow = 0x80000000;
uint32_t nmeaHgtAvLow = 0x80000000;

int  NmeaLatAv = 0;
int  NmeaLngAv = 0;
int  NmeaHgtAv = 0;

int  NmeaHeightDevAboveMsl;
int  NmeaHeightDevAboveMslAv;
int  NmeaHeightMslAboveWgs;

int  NmeaDop; //In 100ths: dop < 2 is good; dop > 5 is poor.
int  NmeaSatelliteCount;
int  NmeaFixQuality;

void NmeaDegToString(char plus, char minus, int deg, char *text)
{
    int units    = labs(deg) / 10000000;
    int fraction = labs(deg) % 10000000;
    char dir = deg >= 0 ? plus : minus;
    sprintf(text, "%c%d.%05d", dir,   units,  fraction / 100);
}
void NmeaHgtToString(char plus, char minus, int hgt, char *text)
{
    int units    = labs(hgt) / 1000000;
    int fraction = labs(hgt) % 1000000;
    char dir = hgt >= 0 ? plus : minus;
    sprintf(text, "%c%d.%d", dir,   units,  fraction / 100000);
}
void NmeaDopToString(int dop, char* text)
{    //In 100ths
    int units    = dop / 100;
    int fraction = dop % 100;
    sprintf(text, "%d.%d", units, fraction);
}
static void addAv(int* pAccH, uint32_t* pAccL, int val, int shift)
{
    int64_t acc =  ((int64_t)*pAccH << 32) | *pAccL;
    int64_t diff = ((int64_t)val    << 32) - acc;
    int64_t add = diff >> shift;
    acc += add;
    *pAccH = acc >> 32;
    *pAccL = acc & 0xFFFFFFFF;
}

static uint32_t msTimerMsgWatchdog = (uint32_t)-TIMEOUT_MS;
static bool nmeaWasStopped = true;
void NmeaFixReceived()
{
    if (nmeaWasStopped) GpsLog("NMEA fix messages have started\r\n");
    nmeaWasStopped = false;
    msTimerMsgWatchdog = MsTimerCount; //Feed the watchdog whenever a fix message comes in
    
    static bool lastHaveFix = false;
    
    if (NmeaFixQuality)
    {
        confidence++;
        if (confidence > CONFIDENCE_OK) confidence = CONFIDENCE_OK;
        if (!lastHaveFix) GpsLog("NMEA fix acquired\r\n");
        lastHaveFix = true;
    }
    else
    {
        confidence = 0;
        if (lastHaveFix) GpsLog("NMEA fix lost - sync disabled\r\n");
        lastHaveFix = false;
    }

    NmeaHgt = NmeaHeightDevAboveMsl + NmeaHeightMslAboveWgs;
    
    if (confidence == CONFIDENCE_OK)
    {
        addAv(&NmeaLatAv, &nmeaLatAvLow, NmeaLat, AVERAGE_SHIFT_DIVISOR);
        addAv(&NmeaLngAv, &nmeaLngAvLow, NmeaLng, AVERAGE_SHIFT_DIVISOR);
        addAv(&NmeaHgtAv, &nmeaHgtAvLow, NmeaHgt, AVERAGE_SHIFT_DIVISOR);
    }
    
    NmeaHeightDevAboveMslAv = NmeaHgtAv - NmeaHeightMslAboveWgs;
}
static void checkIfReceivingNmeaFixMessages()
{    
    if (MsTimerRelative(msTimerMsgWatchdog, TIMEOUT_MS))
    {
        confidence = 0;
        if (!nmeaWasStopped) GpsLog("NMEA fix messages have stopped - sync disabled\r\n");
        nmeaWasStopped = true;
    }
}
static void checkConfidence()
{
    static bool wasConfident = false;
    if (confidence == CONFIDENCE_OK)
    {
        if (!wasConfident) GpsLog("NMEA fix is stable\r\n");
        wasConfident = true;
    }
    else
    {
        if (wasConfident) GpsLog("NMEA fix is not stable\r\n");
        wasConfident = false;
    }
}
void NmeaFixMain()
{
    checkIfReceivingNmeaFixMessages();
    checkConfidence();
}